// Copyright (C) 2000 TerraSoft Ltd
// All rights reserved.

// Limitations: cannot be used with a list of type 'bool' because of
//              the defined operator bool()

#ifndef _LLIST_ITER
#define _LLIST_ITER

// interoperability with internal software library's
// linked list and MFC's CList
#ifndef __TLList_
#   define TLList CList
#endif  // __TLList_

template<class TYPE, class ARG_TYPE>
class TLListIter
{
  private:
    POSITION                m_posCurrent;
    TLList<TYPE, ARG_TYPE> &m_list;

  protected:
    // for use in derived classes only
    void SetPosition(POSITION &pos);

  public:
    // construction/destruction
    TLListIter(TLList<TYPE, ARG_TYPE> &__list);
    TLListIter(TLListIter<TYPE, ARG_TYPE> &__iter);
    virtual ~TLListIter();

    // incrementer/decrementer
    TLListIter<TYPE, ARG_TYPE> operator++(int); // post increment
    TLListIter<TYPE, ARG_TYPE> operator--(int); // post decrement

    // operators
    operator bool ();       // return true if there is a current element
    operator ARG_TYPE ();   // returns the current element

    // these functions will return false if there are no elements in the list
    bool begin(void);       // move to the beginning of the list
    bool end(void);         // move to the end of the list

    POSITION GetPosition(void) const;
};

// basic constructor
template<class TYPE, class ARG_TYPE>
TLListIter<TYPE, ARG_TYPE>::TLListIter<TYPE, ARG_TYPE>(TLList<TYPE, ARG_TYPE> &__list)
  : m_list(__list)
{
    m_posCurrent = m_list.GetHeadPosition();
}

// copy constructor
template<class TYPE, class ARG_TYPE>
TLListIter<TYPE, ARG_TYPE>::TLListIter<TYPE, ARG_TYPE>(TLListIter<TYPE, ARG_TYPE> &__iter)
  : m_list(__iter.m_list)
{
    m_posCurrent = __iter.m_posCurrent;
}

// destructor
template<class TYPE, class ARG_TYPE>
TLListIter<TYPE, ARG_TYPE>::~TLListIter<TYPE, ARG_TYPE>()
{
}

template<class TYPE, class ARG_TYPE>
void TLListIter<TYPE, ARG_TYPE>::SetPosition(POSITION &pos)
{
    m_posCurrent = pos;
}

// post increment
template<class TYPE, class ARG_TYPE>
TLListIter<TYPE, ARG_TYPE> TLListIter<TYPE, ARG_TYPE>::operator++(int)
{
    TLListIter<TYPE, ARG_TYPE> iter(*this);
    m_list.GetNext(m_posCurrent);
    return iter;
}

// post decrement
template<class TYPE, class ARG_TYPE>
TLListIter<TYPE, ARG_TYPE> TLListIter<TYPE, ARG_TYPE>::operator--(int)
{
    TLListIter<TYPE, ARG_TYPE> iter(*this);
    m_list.GetPrev(m_posCurrent);
    return iter;
}

// return true if is there is a current element
template<class TYPE, class ARG_TYPE>
TLListIter<TYPE, ARG_TYPE>::operator bool ()
{
    return (m_posCurrent != NULL);
}

// return the current element
template<class TYPE, class ARG_TYPE>
TLListIter<TYPE, ARG_TYPE>::operator ARG_TYPE ()
{
    return m_list.GetAt(m_posCurrent);
}

// move to the beginning of the list
template<class TYPE, class ARG_TYPE>
bool TLListIter<TYPE, ARG_TYPE>::begin(void)
{
    m_posCurrent = m_list.GetHeadPosition();
    return (m_posCurrent != NULL);
}

// move to the end of the list
template<class TYPE, class ARG_TYPE>
bool TLListIter<TYPE, ARG_TYPE>::end(void)
{
    m_posCurrent = m_list.GetTailPosition();
    return (m_posCurrent != NULL);
}

// returns the POSITION of the current element
template<class TYPE, class ARG_TYPE>
POSITION TLListIter<TYPE, ARG_TYPE>::GetPosition(void) const
{
    return m_posCurrent;
}

#endif // _LLIST_ITER
